'use strict';
const ManagerController = require('../manager.js');
/**
 * [exports 产品表]
 * @type {[type]}
 */
module.exports = class ProductsController extends ManagerController {
  /**
   * [useModel 使用模型]
   * @return {[type]} [description]
   */
  get useModel() {
    const that = this;
    return that.app.model.Products;
  }

  /**
   * [useContentModel 商品详情表]
   * @return {[type]} [description]
   */
  get useContentModel() {
    const that = this;
    return that.app.model.ProductContents;
  }

  /**
   * [useCarouseModel 商品相册表]
   * @return {[type]} [description]
   */
  get useCarouseModel() {
    const that = this;
    return that.app.model.ProductCarousels;
  }

  /**
   * [createValidate 添加产品验证器]
   * @return {[type]} [description]
   */
  get createValidate() {
    const that = this;
    return {
      product_name: that.ctx.rules.name('商品名称')
        .required()
        .notEmpty()
        .trim(),
      product_desction: that.ctx.rules.default('')
        .required(),
      product_stock: that.ctx.rules.default(99)
        .required()
        .number(),
      product_image: that.ctx.rules.default('')
        .required(),
      shop_price: that.ctx.rules.name('商品价格')
        .required()
        .notEmpty()
        .number(),
      market_price: that.ctx.rules.default(0)
        .number()
        .extend((field, value, row) => {
          if (!value) row[field] = row.shop_price + row.shop_price * 1.5;
        }),
      cost_price: that.ctx.rules.default(0)
        .number(),
      activity_id: that.ctx.rules.default(0)
        .number(),
      carousel_id: that.ctx.rules.default(0)
        .number()
        .extend(async (field, value, row) => {
          if (row.content) {
            const tmpContent = await that.ctx.postParse('content', '', val => val);
            const result = await that.useCarouseModel.create({
              content: tmpContent,
              admin_id: that.service.manager.ActionAdminUserId(),
              create_time: that.app.szjcomo.date('Y-m-d H:i:s'),
            });
            if (result) row[field] = result.carousel_id;
          }
        }),
      category_id: that.ctx.rules.name('商品分类')
        .required()
        .notEmpty()
        .number(),
      product_sn: that.ctx.rules.default('')
        .extend((field, value, row) => {
          if (!value) row[field] = `${that.app.szjcomo.date('YmdHis')}-${that.app.szjcomo.mt_rand(100000, 999999)}`;
        }),
      supplier_id: that.ctx.rules.default(0)
        .number(),
      is_sale: that.ctx.rules.default(1)
        .number(),
      alert_stock: that.ctx.rules.default(0)
        .number(),
      goods_type: that.ctx.rules.default(1)
        .number(),
      article_id: that.ctx.rules.default(0)
        .number(),
      is_new: that.ctx.rules.default(0)
        .number(),
      is_hot: that.ctx.rules.default(0)
        .number(),
      is_best: that.ctx.rules.default(0)
        .number(),
      content_id: that.ctx.rules.default(0)
        .required()
        .extend(async (field, value, row) => {
          if (row.desction) {
            const tmpDesction = await that.ctx.postParse('desction', '', val => val);
            const result = await that.useContentModel.create({
              desction: tmpDesction,
              admin_id: that.service.manager.ActionAdminUserId(),
              create_time: that.app.szjcomo.date('Y-m-d H:i:s'),
            });
            if (result) row[field] = result.content_id;
          }
        }),
      admin_id: that.ctx.rules.default(that.service.manager.ActionAdminUserId())
        .number(),
      create_time: that.ctx.rules.default(that.app.szjcomo.date('Y-m-d H:i:s'))
        .required(),
    };
  }

  /**
   * [updateValidate 更新商品]
   * @return {[type]} [description]
   */
  get updateValidate() {
    const that = this;
    return {
      product_id: that.ctx.rules.name('商品ID')
        .required()
        .notEmpty()
        .number()
        .extend(async (field, value, row) => {
          if (row.carousel_id) {
            const tmpContent = await that.ctx.postParse('content', '', val => val);
            const result = await that.useCarouseModel.update({
              content: tmpContent,
              admin_id: that.service.manager.ActionAdminUserId(),
              update_time: that.app.szjcomo.date('Y-m-d H:i:s'),
            }, {
              where: { carousel_id: row.carousel_id },
            });
            if (!result) return '商品相册更新失败,请重试';
          }
          if (row.content_id) {
            const tmpDesction = await that.ctx.postParse('desction', '', val => val);
            const result = await that.useContentModel.update({
              desction: tmpDesction,
              admin_id: that.service.manager.ActionAdminUserId(),
              update_time: that.app.szjcomo.date('Y-m-d H:i:s'),
            }, {
              where: { content_id: row.content_id },
            });
            if (!result) return '商品详情更新失败,请重试';
          }
        }),
      admin_id: that.ctx.rules.default(that.service.manager.ActionAdminUserId())
        .number(),
      update_time: that.ctx.rules.default(that.app.szjcomo.date('Y-m-d H:i:s'))
        .required(),
    };
  }

  /**
   * [selectValidate 查询验证器]
   * @return {[type]} [description]
   */
  get selectValidate() {
    const that = this;
    return {
      page: that.ctx.rules.default(1)
        .number(),
      limit: that.ctx.rules.default(20)
        .number(),
      product_name: that.ctx.rules.default('')
        .required(),
      supplier_id: that.ctx.rules.default(0)
        .number(),
      is_new: that.ctx.rules.default('')
        .required(),
      is_hot: that.ctx.rules.default('')
        .required(),
      is_best: that.ctx.rules.default('')
        .required(),
      category_id: that.ctx.rules.default(0)
        .number(),
      product_type: that.ctx.rules.default(0)
        .number(),
      alert_stock: that.ctx.rules.default('')
        .required(),
      is_sale: that.ctx.rules.default('')
        .required(),
      product_id: that.ctx.rules.default(0)
        .number(),
    };
  }

  /**
   * [desctionValidate 查询商品详情验证器]
   * @return {[type]} [description]
   */
  get desctionValidate() {
    const that = this;
    return {
      content_id: that.ctx.rules.name('商品详情ID')
        .required()
        .number(),
    };
  }

  /**
   * [carouselsValidate 查询商品相册验证器]
   * @return {[type]} [description]
   */
  get carouselsValidate() {
    const that = this;
    return {
      carousel_id: that.ctx.rules.name('商品相册ID')
        .required()
        .number(),
    };
  }

  /**
   * [create 添加商品]
   * @return {[type]} [description]
   */
  async create() {
    const that = this;
    let transaction;
    try {
      const data = await that.ctx.validate(that.createValidate, await that.ctx.postParse());
      transaction = await that.app.model.transaction();
      const createBean = await that.app.comoBean.instance(data, {
        transaction,
      });
      const result = await that.service.manager.create(createBean, that.useModel, '商品添加失败,请稍候重试');
      await that.productActionAfter(result.product_id, transaction);
      await transaction.commit();
      return that.ctx.appJson(that.app.szjcomo.appResult('SUCCESS', result, false));
    } catch (err) {
      if (transaction) await transaction.rollback();
      return that.ctx.appJson(that.app.szjcomo.appResult(err.message));
    }
  }

  /**
   * [createAfter 添加商品后缀操作]
   * @param  {[type]} data       [description]
   * @param  {[type]} product_id [description]
   * @return {[type]}            [description]
   */
  async productActionAfter(product_id, transaction) {
    const that = this;
    const spes = await that.ctx.postParse('spes', [], val => val);
    if (spes && spes.length > 0) {
      await that.app.model.ProductSpes.destroy({ where: { product_id }, transaction });
      const datas = spes.map(item => {
        return Object.assign(item, {
          product_id,
          admin_id: that.service.manager.ActionAdminUserId(),
          create_time: that.app.szjcomo.date('Y-m-d H:i:s'),
        });
      });
      const res = await that.app.model.ProductSpes.bulkCreate(datas, { transaction });
      if (!res) throw new Error('商品属性操作失败,请重试');
    }
  }

  /**
   * [update 更新商品]
   * @return {[type]} [description]
   */
  async update() {
    const that = this;
    let transaction;
    try {
      const data = await that.ctx.validate(that.updateValidate, await that.ctx.anyParse());
      transaction = await that.app.model.transaction();
      const updateBean = await that.app.comoBean.instance(data, {
        where: { product_id: data.product_id }, transaction,
      });
      const result = await that.service.manager.update(updateBean, that.useModel, '商品更新失败,请稍候重试');
      await that.productActionAfter(data.product_id, transaction);
      await transaction.commit();
      return that.ctx.appJson(that.app.szjcomo.appResult('SUCCESS', result, false));
    } catch (err) {
      if (transaction) transaction.rollback();
      return that.ctx.appJson(that.app.szjcomo.appResult(err.message));
    }
  }

  /**
   * [select 查询商品列表]
   * @return {[type]} [description]
   */
  async select() {
    const that = this;
    try {
      const data = await that.ctx.validate(that.selectValidate, await that.ctx.getParse());
      const seq = that.app.Sequelize;
      const options = {
        offset: (data.page - 1) * data.limit, limit: data.limit, where: {}, include: [
          { model: that.app.model.AdminUser, as: 'admin_user', attributes: [] },
          { model: that.app.model.ProductCategory, as: 'categorys', attributes: [] },
        ], attributes: {
          exclude: [ 'admin_id', 'brand_id' ],
          include: [
            [ seq.col('admin_user.username'), 'username' ],
            [ seq.col('categorys.category_name'), 'category_name' ],
          ],
        }, order: [[ 'product_id', 'desc' ]],
      };
      if (data.category_id) options.where.category_id = data.category_id;
      if (data.carousel_id) options.where.carousel_id = data.carousel_id;
      if (data.product_name) options.where.product_name = { [seq.Op.regexp]: data.product_name };
      if (data.is_hot !== '') options.where.is_hot = Number(data.is_hot);
      if (data.is_new !== '') options.where.is_new = Number(data.is_new);
      if (data.is_best !== '') options.where.is_best = Number(data.is_best);
      if (data.is_sale !== '') options.where.is_sale = Number(data.is_sale);
      if (data.product_type) options.where.product_type = data.product_type;
      if (data.supplier_id) options.where.supplier_id = data.supplier_id;
      const selectBean = that.app.comoBean.instance(data, options);
      const result = await that.service.manager.select(selectBean, that.useModel, '商品列表查看失败,请重试', true);
      return that.ctx.appJson(that.app.szjcomo.appResult('SUCCESS', result, false));
    } catch (err) {
      return that.ctx.appJson(that.app.szjcomo.appResult(err.message));
    }
  }

  /**
   * [selectDesction 查询商品详情]
   * @return {[type]} [description]
   */
  async selectDesction() {
    const that = this;
    try {
      const data = await that.ctx.validate(that.desctionValidate, await that.ctx.getParse());
      const info = await that.app.model.ProductContents.findOne({
        where: { content_id: data.content_id },
        attributes: [ 'desction' ],
      });
      return that.ctx.appJson(that.app.szjcomo.appResult('SUCCESS', info, false));
    } catch (err) {
      return that.ctx.appJson(that.app.szjcomo.appResult(err.message));
    }
  }

  /**
   * [selectCarousels 查询商品相册]
   * @return {[type]} [description]
   */
  async selectCarousels() {
    const that = this;
    try {
      const data = await that.ctx.validate(that.carouselsValidate, await that.ctx.getParse());
      const info = await that.app.model.ProductCarousels.findOne({
        where: { carousel_id: data.carousel_id },
        attributes: [ 'content' ],
      });
      return that.ctx.appJson(that.app.szjcomo.appResult('SUCCESS', info, false));
    } catch (err) {
      return that.ctx.appJson(that.app.szjcomo.appResult(err.message));
    }
  }

  /**
   * [selectSpes 获取商品属性]
   * @return {[type]} [description]
   */
  async selectSpes() {
    const that = this;
    try {
      const seq = that.app.Sequelize;
      const data = await that.ctx.validate({
        product_id: that.ctx.rules.name('商品ID')
          .required()
          .number(),
      }, await that.ctx.getParse());
      const list = await that.app.model.ProductSpes.findAll({
        where: { product_id: data.product_id },
        include: [
          { model: that.app.model.ProductTypesItem, as: 'product_types_item', attributes: [] },
        ],
        attributes: {
          exclude: [ 'admin_id', 'create_time', 'update_time', 'spe_id', 'product_id' ],
          include: [
            [ seq.col('product_types_item.item_name'), 'item_name' ],
            [ seq.col('product_types_item.item_value'), 'item_value' ],
            [ seq.col('product_types_item.item_type'), 'item_type' ],
          ],
        },
      });
      return that.ctx.appJson(that.app.szjcomo.appResult('SUCCESS', list, false));
    } catch (err) {
      return that.ctx.appJson(that.app.szjcomo.appResult(err.message));
    }
  }

};
